#pragma once

#include <vector>

#include "glm/glm.hpp"

typedef std::vector<double> Vector6;

class Orbit
{
public:
    Orbit();
    ~Orbit() = default; 

    double getSemiMajorAxis() const;
    double getEccentricity() const;
    double getInclination() const;
    double getArgumentOfPeriapsis() const;
    double getLongitudeOfAscendingNode() const;

    void setSemiMajorAxis(double semiMajorAxis);
    void setEccentricity(double eccentricity);
    void setInclination(double inclination);
    void setArgumentOfPeriapsis(double argumentOfPeriapsis);
    void setLongitudeOfAscendingNode(double longitudeOfAscendingNode);

    glm::dvec3 getPosition(double gravitationalParameter, double time) const;
    glm::dvec3 getPositionFromMeanAnomaly(double meanAnomaly, double gravitationalParameter) const;

    glm::dvec3 getVelocity(double gravitationalParameter, double time) const;
    glm::dvec3 getVelocityFromMeanAnomaly(double meanAnomaly, double gravitationalParameter) const;

    //glm::dvec3 getTangent(const double meanAnomaly) const;
    //glm::mat4 getLookAlongMatrix(const double meanAnomaly) const;

    const Vector6 getCartesianCoordinates(const double meanAnomaly, const double gravitationalParameter) const;
    const double getMeanAnomaly(const double gravitationalParameter, const double time) const;

private:
    Vector6 _keplerianElements;

    const double getEccentricAnomaly(const double meanAnomaly) const;
    const double getTrueAnomaly(const double meanAnomaly) const;
};
